/* $Id: compat.h,v 1.6 2000/08/09 20:34:41 mikpe Exp $
 * Performance-monitoring counters driver.
 * Compatibility definitions for 2.2.x/2.3.x/2.4.x kernels.
 *
 * Copyright (C) 1999-2000  Mikael Pettersson
 */
#include <linux/fs.h>
#include <linux/version.h>

#if !defined(virt_to_page)	/* kernel < 2.4.0-test6 */
#define virt_to_page(kaddr)	(mem_map + MAP_NR(kaddr))
#endif

#if !defined(fops_get)		/* kernel < 2.4.0-test2 */
#define fops_get(fops)		(fops)
#define OWNER_THIS_MODULE	/*empty*/
#define MOD_INC_OPEN(func)	.open = func,
#define MOD_DEC_RELEASE(func)	.release = func,
#define NEED_MOD_INC_OPEN	/*XXX: also for 2.4.0-test2*/
#else
#define OWNER_THIS_MODULE	.owner = THIS_MODULE,
#define MOD_INC_OPEN(func)	/*empty*/
#define MOD_DEC_RELEASE(func)	/*empty*/
#endif

#if !defined(PT_PTRACED)	/* kernel < 2.4.0-test2 */
#define TASK_IS_PTRACED(tsk)	((tsk)->flags & PF_PTRACED)
#else
#define TASK_IS_PTRACED(tsk)	((tsk)->ptrace & PT_PTRACED)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,99)
/* XXX: < 2.3.99-pre7, but KERNEL_VERSION cannot express that :-( */
#define perfctr_get_task_struct(tsk)	do{}while(0)
#define perfctr_free_task_struct(tsk)	do{}while(0)
#else
#define perfctr_get_task_struct(tsk)	get_task_struct((tsk))
#define perfctr_free_task_struct(tsk)	free_task_struct((tsk))
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,99)
/* XXX: < 2.3.99-pre2, but KERNEL_VERSION cannot express that :-( */
#define MAYBE_EXPORT_pidhash		EXPORT_SYMBOL(pidhash)
#define MAYBE_EXPORT_tasklist_lock	EXPORT_SYMBOL(tasklist_lock)
#else
#define MAYBE_EXPORT_pidhash		/*empty*/
#define MAYBE_EXPORT_tasklist_lock	/*empty*/
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,25)
#define vma_pgoff(vma)	((vma)->vm_offset) /* NOT, but suffices for != 0 */
#else
#define vma_pgoff(vma)	((vma)->vm_pgoff)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,23)
#define get_zeroed_page(mask)	get_free_page((mask))
#define SetPageReserved(page)	set_bit(PG_reserved, &(page)->flags)
#define ClearPageReserved(page)	clear_bit(PG_reserved, &(page)->flags)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,13)
#define ARRAY_SIZE(x)	(sizeof(x) / sizeof((x)[0]))
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,13)
#ifdef MODULE
#define __exit		/* empty */
#define module_init(x)	int init_module(void) { return x(); }
#define module_exit(x)	void cleanup_module(void) { x(); }
#else
#define __exit		__attribute__((unused, __section__("text.init")))
#define module_init(x)	/* explicit call is needed */
#define module_exit(x)	/* empty */
#endif
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,13) && defined(MODULE)
#define __cacheline_aligned __attribute__((__aligned__(SMP_CACHE_BYTES)))
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,11)
#define X86_CR4_TSD	0x0004
#define X86_CR4_PCE	0x0100
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,11)
#define TASK_VPERFCTR(tsk)	((tsk)->tss.perfctr)
#else
#define TASK_VPERFCTR(tsk)	((tsk)->thread.perfctr)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,9)
#define get_file(x)	((x)->f_count++)
#define file_count(x)	((x)->f_count)
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2,3,10)
#define get_file(x)	atomic_inc(&(x)->f_count)
#define file_count(x)	atomic_read(&(x)->f_count)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,6)
static __inline__ struct dentry *my_d_alloc_root(struct inode *inode)
{
	return d_alloc_root(inode, NULL);
}
#undef d_alloc_root	/* in case CONFIG_MODVERSIONS renamed it */
#define d_alloc_root(inode)	my_d_alloc_root((inode))
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,1)
#define DECLARE_MUTEX(name)	struct semaphore name = MUTEX
#endif
